@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: checkout.cc 1.16.1.24.1.3.1.8.1.1.1.25 Tue, 06 Oct 1998 13:31:13 -0700 jmacd $
+ * $Id: checkout.cc 1.16.1.24.1.3.1.8.1.1.1.27 Sun, 03 Feb 2002 00:07:41 +0300 jmacd $
  */
 
 #include "prcs.h"
@@ -131,8 +131,9 @@ PrPrcsExitStatusError checkout_command()
 static PrPrcsExitStatusError create_empty_project()
 {
     if (strcmp(cmd_version_specifier_major, "@") == 0 ||
-        cmd_version_specifier_major[0] == '\0')
+        cmd_version_specifier_major[0] == '\0') {
         cmd_version_specifier_major = "0";
+    }
 
     ProjectDescriptor* new_project;
 
@@ -392,7 +393,7 @@ static PrVoidError checkout_keywords(FileEntry* fe)
 
 	    checksum = md5_buffer(unsetkey_buffer.segment(), unsetkey_buffer.length());
 
-	    if(strncmp(checksum, version_data->unkeyed_checksum(), 16) == 0) {
+	    if(memcmp(checksum, version_data->unkeyed_checksum(), 16) == 0) {
 
 		if(old_file_sets_keys) {
 		    /* Files are the same modulo keywords */
@@ -433,8 +434,9 @@ static PrVoidError checkout_keywords(FileEntry* fe)
 
 	/* version_data->length() is an optimization to avoid resizing the
 	 * buffer each time the pipe data doubles in size */
-	If_fail(unsetkey_buffer.map_file(fileno(cofile), version_data->length()))
-	    pthrow prcserror << "Read failed from RCS co pipe" << perror;
+	Dstring ds;
+	fe->describe (ds);
+	Return_if_fail(unsetkey_buffer.map_file(ds.cast (), fileno(cofile), version_data->length()));
 
 	Return_if_fail(VC_close_checkout_stream(cofile,
 						fe->descriptor_version_number(),
@@ -465,8 +467,7 @@ static PrVoidError checkout_keywords(FileEntry* fe)
 	if(new_setkey_len == fe->stat_length()) {
 	    /* Length is the same, now try memcmp */
 
-	    If_fail(file_buffer.map_file(filename))
-		pthrow prcserror << "Read error on " << squote(filename) << perror;
+	    Return_if_fail(file_buffer.map_file(filename));
 
 	    if(memcmp(new_setkey_buf, file_buffer.segment(), new_unsetkey_len) == 0) {
 		/* No differences. */
@@ -588,12 +589,11 @@ static PrVoidError checkout_nokeywords(FileEntry* fe)
 
 	if(version_data->length() == fe->stat_length()) {
 
-	    If_fail(file_buffer.map_file(filename))
-		pthrow prcserror << "Read error on " << squote(filename) << perror;
+	    Return_if_fail(file_buffer.map_file(filename));
 
 	    char* checksum = md5_buffer(file_buffer.segment(), file_buffer.length());
 
-	    if(strncmp(checksum, version_data->unkeyed_checksum(), 16) == 0) {
+	    if(memcmp(checksum, version_data->unkeyed_checksum(), 16) == 0) {
 		if(option_long_format)
 		    prcsoutput << "File " << squote(filename) << " is unmodified" << dotendl;
 
