@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: diff.cc 1.20.1.19.1.3.1.6.1.21 Sun, 31 Oct 1999 16:52:32 -0800 jmacd $
+ * $Id: diff.cc 1.20.1.19.1.3.1.6.1.22 Mon, 18 Sep 2000 04:56:26 +0400 jmacd $
  */
 
 #include "prcs.h"
@@ -324,8 +324,8 @@ static PrIntError try_optimizations(FileEntry *tofe, FileEntry *fromfe)
     if(diff_option_brief && !use_working_to_file &&
        !need_to_replace_to_keywords && !need_to_replace_from_keywords) {
 
-	if(strncmp(to_version_data->unkeyed_checksum(),
-		   from_version_data->unkeyed_checksum(), 16) == 0) {
+	if(memcmp(to_version_data->unkeyed_checksum(),
+		  from_version_data->unkeyed_checksum(), 16) == 0) {
 	    return NODIFFS;
 	} else {
 	    prcsoutput << "The file " << diff_tuple(fromfe, tofe) << " differs" << prcsendl;
@@ -348,9 +348,7 @@ static PrIntError try_optimizations(FileEntry *tofe, FileEntry *fromfe)
 					  tofe,
 					  Unsetkeys));
 	} else {
-	    If_fail(setkey_buf.map_file(tofe->working_path()))
-		pthrow prcserror << "Read failure on " << squote(tofe->working_path())
-				<< perror;
+	    Return_if_fail(setkey_buf.map_file(tofe->working_path()));
 	}
 
 	if(setkey_buf.length() == from_version_data->length()) {
@@ -358,7 +356,7 @@ static PrIntError try_optimizations(FileEntry *tofe, FileEntry *fromfe)
 
 	    checksum = md5_buffer(setkey_buf.segment(), setkey_buf.length());
 
-	    if(strncmp(checksum, from_version_data->unkeyed_checksum(), 16) == 0)
+	    if(memcmp(checksum, from_version_data->unkeyed_checksum(), 16) == 0)
 		return NODIFFS;
 	}
 
@@ -397,7 +395,17 @@ static PrVoidError prepare_diff_versionfile(FileEntry *fe, bool use_temp_file_fr
 	Return_if_fail(cofile << VC_checkout_stream(fe->descriptor_version_number(),
 						    fe->full_descriptor_name()));
 
-	Return_if_fail(setkeys_infile(fileno(cofile), filename, fe, Setkeys));
+	// Blah
+
+	Dstring ds;
+	fe->describe (ds);
+
+	RcsVersionData *fe_data;
+	Return_if_fail(fe_data << fe->project()->repository_entry() ->
+		       lookup_rcs_file_data(fe->descriptor_name(),
+					    fe->descriptor_version_number()));
+
+	Return_if_fail(setkeys_infile(ds.cast (), fileno(cofile), fe_data->length (), filename, fe, Setkeys));
 
 	Return_if_fail(VC_close_checkout_stream(cofile,
 						fe->descriptor_version_number(),
