@@ -16,15 +16,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: dstring.cc 1.2.1.3.1.1.1.6 Tue, 13 Oct 1998 13:28:35 -0700 jmacd $
+ * $Id: dstring.cc 1.2.1.3.1.1.1.6.2.2 Tue, 05 Feb 2002 18:58:12 +0300 jmacd $
  */
 
-#include "dstring.h"
-#include "config.h"
-#ifdef HAVE_STD_H
-#include <std.h>
-#endif
-#include <stdarg.h>
+#include "prcs.h"
 
 #if defined(__GNUG__) || defined(__MWERKS__)
 #if defined(__GNUG__)
@@ -35,8 +30,6 @@
 static ostrstream sprintfbuf;
 #endif
 
-#include "utils.h"
-
 ostream& operator<<(ostream& os, const Dstring* S)
 {
     return os << S->cast();
@@ -72,7 +65,7 @@ void Dstring::sprintfa(const char* fmt, ...)
 
     sprintfbuf.vform(fmt, args);
 
-    sprintfbuf << ends;
+    sprintfbuf << '\0';
 
     append(sprintfbuf.str());
 
@@ -96,7 +89,7 @@ void Dstring::sprintf(const char* fmt, ...)
 
     sprintfbuf.vform(fmt, args);
 
-    sprintfbuf << ends;
+    sprintfbuf << '\0';
 
     append(sprintfbuf.str());
 
