@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: populate.h 1.4.1.7.1.1.1.2 Sat, 19 Apr 1997 19:38:46 -0700 jmacd $
+ * $Id: populate.h 1.4.1.7.1.1.1.6 Wed, 03 Jun 1998 02:21:45 +0400 jmacd $
  */
 
 
@@ -24,20 +24,24 @@
 #define _POPULATE_H_
 
 extern "C" {
-#include "regex.h"
+#include "regex2.h"
 }
 
 class PrVoidError;
 
-extern PrVoidError prcs_compile_regex(const char* pat, regex_t *reg);
-extern bool prcs_regex_matches(const char* pat, regex_t *reg);
+extern PrVoidError prcs_compile_regex(const char* pat, reg2ex2_t *reg2);
+extern bool prcs_regex_matches(const char* pat, reg2ex2_t *reg2);
+
+class FileEntry;
 
 class FileRecord {
 public:
     FileRecord() { }
-    FileRecord(FileType t0, Dstring* n0) :type(t0), name(n0) { }
+    FileRecord(FileType t0, Dstring* n0) :type(t0), name(n0), fe(NULL) { }
+    FileRecord(FileType t0, Dstring* n0, FileEntry* fe0) :type(t0), name(n0), fe(fe0) { }
     FileType type;
     Dstring *name;
+    FileEntry *fe; /* Used for deletion */
 };
 
 #endif
