@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: prcs.h 1.16.1.15.1.6.1.12.1.19 Tue, 02 Jun 1998 14:07:47 -0700 jmacd $
+ * $Id: prcs.h 1.16.1.15.1.6.1.12.1.21.1.2 Thu, 07 Feb 2002 07:57:16 +0300 jmacd $
  */
 
 
@@ -41,19 +41,9 @@ extern "C" {
 #include "utils.h"
 }
 
-/* EGCS doesn't have this header. */
-#ifdef HAVE_STD_H
-#include <std.h>
-#endif
-
 #include <iostream.h>
 #include <fstream.h>
 
-#ifdef __GNUG__
-/* This gets defined in config.h now. */
-/*#define PRCS_DEVEL*/
-#endif
-
 #ifdef NULL
 #undef NULL
 #endif
@@ -113,6 +103,8 @@ extern int option_pipe;                    /* --pipe */
 extern int option_skilled_merge;           /* -s */
 extern int option_plain_format;            /* --plain-format */
 extern int option_sort;                    /* --sort */
+extern int option_nokeywords;              /* --no-keywords */
+extern int option_version_log;             /* --version-log */
 #ifdef PRCS_DEVEL
 #define option_debug (! option_n_debug)    /* --debug */
 extern int option_n_debug;                 /* ! --debug */
@@ -122,9 +114,12 @@ extern int option_tune;                    /* --tune */
 extern const char *option_match_file_pattern;
 extern const char *option_not_match_file_pattern;
 extern const char *option_sort_type;
+extern const char *option_version_log_string;
 
 extern const int prcs_version_number[3];
 
+extern const char* maintainer;
+
 extern const char* temp_file_1;
 extern const char* temp_file_2;
 extern const char* temp_file_3;
@@ -136,6 +131,7 @@ extern PrPrcsExitStatusError depopulate_command();         /* populate.cc */
 extern PrPrcsExitStatusError diff_command();               /* diff.cc */
 extern PrPrcsExitStatusError execute_command();            /* execute.cc */
 extern PrPrcsExitStatusError info_command();               /* info.cc */
+extern PrPrcsExitStatusError changes_command();            /* changes.cc */
 extern PrPrcsExitStatusError package_command();            /* package.cc */
 extern PrPrcsExitStatusError unpackage_command();          /* package.cc */
 extern PrPrcsExitStatusError merge_command();              /* merge.cc */
