@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: typedefs.h 1.8.1.10.1.6.2.1.1.6.1.9 Tue, 29 Sep 1998 13:26:59 -0700 jmacd $
+ * $Id: typedefs.h 1.8.1.10.1.6.2.1.1.6.1.11 Sun, 28 Oct 2001 10:54:02 +0300 jmacd $
  */
 
 
@@ -94,17 +94,18 @@ enum PrcsExitStatus {
 };
 
 enum MergeAction {
-    MergeActionMerge   = 'm',
-    MergeActionAdd     = 'a',
-    MergeActionDelete  = 'd',
-    MergeActionNothing = 'n',
-    MergeActionReplace = 'r'
+    MergeActionNoPrompt = 0,
+    MergeActionMerge    = 'm',
+    MergeActionAdd      = 'a',
+    MergeActionDelete   = 'd',
+    MergeActionNothing  = 'n',
+    MergeActionReplace  = 'r'
 };
 
 enum FileType {
-    SymLink,
-    Directory,
-    RealFile
+    SymLink = 1,
+    Directory = 2,
+    RealFile = 3
 };
 
 enum MergeParentState {
