@@ -5,7 +5,7 @@
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * of the License, or (at your option) an later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: merge.cc 1.18.1.1.1.26.1.1.1.4.1.9.1.38 Sun, 31 Oct 1999 16:52:32 -0800 jmacd $
+ * $Id: merge.cc 1.18.1.1.1.26.1.1.1.4.1.9.1.42 Tue, 05 Feb 2002 22:49:08 +0300 jmacd $
  */
 
 extern "C" {
@@ -73,7 +73,7 @@ protected:
 
 private:
 
-    virtual MergeAction merge_action (Mergeable* m, int rule) = 0;
+    virtual MergeAction merge_action_by_rule (Mergeable* m, int rule) = 0;
     virtual const char* merge_desc   (Mergeable* m, int rule) = 0;
     virtual PrBoolError quick_cmp    (Mergeable* sel, Mergeable* com, Mergeable* work, bool* positive) = 0;
     virtual PrVoidError prepare      (Mergeable* selected, const char* sel_name,
@@ -227,19 +227,27 @@ extern const char *default_merge_descs[14] =
 
 extern const MergeAction default_merge_actions[14] =
 {
-    (MergeAction) 0,
+    /* There is a problem in 1.3.0 reported by Dan Bonachea not really
+     * having to do with NoPrompt merge actions -- but I'm writing
+     * this comment anyway.  When you do a merge and get noprompt
+     * because your working file is the same as the selected version
+     * and then do a partial checkin, excluding that file, you should
+     * not clobber that version when you checkin to the selected
+     * branch.  Alas, the problem is with partial checkin, not merge.
+     */
+    MergeActionNoPrompt, /* 1 == selected */
     MergeActionMerge,
     MergeActionReplace,
-    (MergeAction) 0,
-    (MergeAction) 0,
+    MergeActionNoPrompt, /* 4 != selected */
+    MergeActionNoPrompt, /* 5 == selected */
     MergeActionNothing,
     MergeActionDelete,
-    (MergeAction) 0,
+    MergeActionNoPrompt, /* 8 == selected */
     MergeActionNothing,
     MergeActionDelete,
     MergeActionMerge,
-    (MergeAction) 0,
-    (MergeAction) 0,
+    MergeActionNoPrompt, /* 12 no selected */
+    MergeActionNoPrompt, /* 13 no selected */
     MergeActionAdd
 };
 
@@ -277,6 +285,9 @@ static const MergeDef merge_defaults[14] =
 #define temp_file_common temp_file_2
 #define temp_file_selected temp_file_3
 
+static bool         any_obsolete = false;
+static CharPtrArray all_conflicts;
+
 static PrVoidError checkout_unkey (FileEntry* selected, const char* sel_name,
 				   FileEntry* common, const char* com_name,
 				   FileEntry* working, const char* work_name,
@@ -372,7 +383,7 @@ protected:
 
 private:
 
-    virtual MergeAction merge_action (Mergeable* m, int rule) {
+    virtual MergeAction merge_action_by_rule (Mergeable* m, int rule) {
 	return CAST(m)->file_attrs()->merge_action(rule - 1);
     }
 
@@ -500,7 +511,7 @@ protected:
 
 private:
 
-    virtual MergeAction merge_action (Mergeable* , int rule) {
+    virtual MergeAction merge_action_by_rule (Mergeable* , int rule) {
 	return default_merge_actions[rule-1];
     }
 
@@ -614,7 +625,7 @@ static KeywordMergeControl* keyword_control = NULL;
 /*			     Main Command                             */
 /**********************************************************************/
 
-PrPrcsExitStatusError merge_command()
+static PrPrcsExitStatusError merge_command2()
 {
     ProjectDescriptor *working, *selected, *common;
     ProjectVersionData *selected_version, *pred_project_data;
@@ -637,6 +648,9 @@ PrPrcsExitStatusError merge_command()
     if(!pred_project_data)
 	pthrow prcserror << "Invalid version in working project file, cannot merge" << dotendl;
 
+    if(pred_project_data->deleted ())
+	pthrow prcserror << "Project version " << pred_project_data << " has been deleted" << dotendl;
+
     eliminate_unnamed_files(working);
 
     Return_if_fail(eliminate_working_files(working, SetNotPresentWarnUser));
@@ -727,8 +741,8 @@ PrPrcsExitStatusError merge_command()
     install_cleanup_handler (merge_cleanup_handler, working);
 
     file_entry_control = new FileMergeControl (working);
-    keyword_control = new KeywordMergeControl (working);
-    populate_control = new PopulateMergeControl (working);
+    keyword_control    = new KeywordMergeControl (working);
+    populate_control   = new PopulateMergeControl (working);
 
     Return_if_fail (merge_common_files(selected, common, working));
     Return_if_fail (merge_selected_only_files(selected, common, working));
@@ -737,11 +751,28 @@ PrPrcsExitStatusError merge_command()
 
     working->merge_notify_complete();
 
+    for (int i = 0; i < all_conflicts.length (); i += 1) {
+	prcsoutput << "File " << squote (all_conflicts.index (i)) << " has conflicts which you must edit" << dotendl;
+    }
+
     /* Note: merged project file is written out by merge_cleanup_handler */
 
     return ExitSuccess;
 }
 
+PrPrcsExitStatusError merge_command()
+{
+    PrPrcsExitStatusError ret = merge_command2 ();
+
+    if (Failure (ret)) {
+
+	if (any_obsolete) {
+	    prcserror << "Merge failed: check `obsolete' subdirectories for saved originals" << dotendl;
+	}
+    }
+
+    return ret;
+}
 
 static PrVoidError merge_cleanup_handler(void* data, bool)
 {
@@ -1297,7 +1328,9 @@ static PrVoidError make_obsolete(FileEntry *fe, ProjectDescriptor *working_proje
     if (force_copy)
       Return_if_fail (fs_copy_filename (work_path, obs));
     else
-      Return_if_fail(fs_move_filename(work_path, obs));
+      Return_if_fail (fs_move_filename (work_path, obs));
+
+    any_obsolete = true;
 
     if(option_long_format)
 	prcsinfo << "Copied working file " << squote(work_path)
@@ -1319,31 +1352,25 @@ static PrBoolError merge_files(ProjectDescriptor* working_project,
     Dstring slabel, clabel, wlabel;
     int ret;
 
+    /* If merging a no_ancestor set of files, common will be null.
+     */
     ASSERT (!common  || common  ->file_type() == RealFile, "check first");
     ASSERT (selected && selected->file_type() == RealFile, "check first");
     ASSERT (working  && working ->file_type() == RealFile, "check first");
-
-    Return_if_fail (make_subdirectories(selected->working_path()));
+    ASSERT (working->present(), "how could this fail?");
 
     Return_if_fail (checkout_unkey (selected, temp_file_selected,
 				    common, temp_file_common,
 				    working, temp_file_working,
 				    co));
 
-    if (working->present())
-	Return_if_fail(working->get_file_sys_info());
-    else
-	Return_if_fail(working->get_repository_info(working->project()->repository_entry()));
+    Return_if_fail(working->get_file_sys_info());
 
     Return_if_fail(selected->get_repository_info(selected->project()->repository_entry()));
 
     if (common)
 	Return_if_fail(common->get_repository_info(common->project()->repository_entry()));
 
-    If_fail(working->stat())
-	pthrow prcserror << "Stat failed on "
-			<< squote(working->working_path()) << perror;
-
     SystemCommand* merge_command = NULL;
 
     if (working->file_attrs()->merge_tool())
@@ -1354,13 +1381,6 @@ static PrBoolError merge_files(ProjectDescriptor* working_project,
     else
 	Return_if_fail (args << gdiff3_command.new_arg_list());
 
-    /*
-     * If merging a no_ancestor set of files, common will be null, so
-     * test the value of the file entry before formatting the label.
-     */
-    ASSERT(selected, "illegal file entry, only common may be null");
-    ASSERT(working,  "illegal file entry, only common may be null");
-
     format_diff_label (selected, selected->working_path(), &slabel);
     format_diff_label (working, working->working_path(), &wlabel);
 
@@ -1412,9 +1432,10 @@ static PrBoolError merge_files(ProjectDescriptor* working_project,
 	Return_if_fail(ret << merge_command->open_stdout());
     }
 
-    if(ret > 1)
+    if (ret > 1) {
 	pthrow prcserror << (merge_command ? merge_command->path() : gdiff3_command.path())
-			<< " exited with status " << ret << dotendl;
+			 << " exited with status " << ret << dotendl;
+    }
 
     Return_if_fail(working->chmod(working->stat_permissions()));
 
@@ -1695,7 +1716,7 @@ static PrBoolError file_quick_cmp  (FileEntry* selected,
 
     *positive = true;
 
-    return strncmp (onedat->unkeyed_checksum(), twodat->unkeyed_checksum(), 16) != 0;
+    return memcmp (onedat->unkeyed_checksum(), twodat->unkeyed_checksum(), 16) != 0;
 }
 
 static PrVoidError maybe_rename (FileEntry* selected,
@@ -1804,19 +1825,6 @@ void MergeControl::merge_announce_os (Mergeable* selected,
     os << "*** Action on " << type() << " " << print3 (selected, common, working) << prcsendl;
 }
 
-void MergeControl::merge_announce (Mergeable* selected,
-				   Mergeable* common,
-				   Mergeable* working,
-				   MergeAction action)
-{
-    /* Yes, I could work some ostream magic and sent one command to the
-     * log and the output... but I choose not to. */
-    merge_announce_os (selected, common, working, project->merge_log());
-
-    if (!(action == (MergeAction)0 || (action == MergeActionNothing && option_force_resolution)) || option_long_format)
-	merge_announce_os (selected, common, working, prcsoutput);
-}
-
 void MergeControl::merge_action_os (Mergeable* selected,
 				    Mergeable* common,
 				    Mergeable* working,
@@ -1849,10 +1857,9 @@ void MergeControl::merge_action_os (Mergeable* selected,
 	break;
     }
 
-    os << desc << " " << type() << " " << print3 (selected, common, working);
+    ASSERT (ruleno != 0, "because no SKIP_FILE");
 
-    if (ruleno)
-	os << " by rule " << ruleno;
+    os << desc << " " << type() << " " << print3 (selected, common, working) << " by rule " << ruleno;
 
     if (conflict)
 	os << ", conflicts created";
@@ -1860,6 +1867,22 @@ void MergeControl::merge_action_os (Mergeable* selected,
     os << prcsendl;
 }
 
+void MergeControl::merge_announce (Mergeable* selected,
+				   Mergeable* common,
+				   Mergeable* working,
+				   MergeAction action)
+{
+    merge_announce_os (selected, common, working, project->merge_log());
+
+    bool skip_noprompt  = (action == MergeActionNoPrompt) && ! option_really_long_format;
+    bool skip_donothing = (action == MergeActionNothing)  && (option_force_resolution || ! option_really_long_format);
+
+    if (! skip_noprompt && ! skip_donothing) {
+
+	merge_announce_os (selected, common, working, prcsoutput);
+    }
+}
+
 void MergeControl::merge_action (Mergeable* selected,
 				 Mergeable* common,
 				 Mergeable* working,
@@ -1869,8 +1892,13 @@ void MergeControl::merge_action (Mergeable* selected,
 {
     merge_action_os (selected, common, working, action, ruleno, conflict, project->merge_log());
 
-    if (!(ruleno == 0 || action == (MergeAction)0 || (action == MergeActionNothing && option_force_resolution)) || option_long_format)
+    bool skip_noprompt  = (action == MergeActionNoPrompt) && ! option_really_long_format;
+    bool skip_donothing = (action == MergeActionNothing)  && (option_force_resolution || ! option_really_long_format);
+
+    if (! skip_noprompt && ! skip_donothing) {
+
 	merge_action_os (selected, common, working, action, ruleno, conflict, prcsoutput);
+    }
 
     project->merge_log() << prcsendl;
 }
@@ -1914,7 +1942,7 @@ MergeAction MergeControl::file_merge_action (Mergeable* selected,
 	ASSERT (non_null, "blah");
     }
 
-    return merge_action (non_null, rule);
+    return merge_action_by_rule (non_null, rule);
 }
 
 PrVoidError MergeControl::finish_merge (Mergeable* selected,
@@ -2053,13 +2081,20 @@ PrVoidError MergeControl::finish_merge (Mergeable* selected,
 	    break;
 	case MergeActionNothing:
 	    break;
+	default:
+	    ASSERT (false, "notreached");
+	    break;
 	}
     }
 
     merge_action (selected, common, working, action, mergedef->rule_no, conflict);
+
     notify (working, common, selected, action);
 
     if (conflict) {
+
+	all_conflicts.append (name (working));
+
 	const char* env = get_environ_var ("PRCS_CONFLICT_EDITOR");
 
 	if (env) {
@@ -2109,10 +2144,10 @@ void MergeControl::print_merge_help(const char* valid)
 }
 
 /* Can skip a rule entirely if this: */
-#define SKIP_FILE(act) ((!option_long_format) && \
-			((act == (MergeAction)0) || \
-			 ((act == MergeActionNothing) && \
-			  (!option_force_resolution))))
+//  #define SKIP_FILE(act) ((!option_long_format) &&
+//  			((act == MergeActionNoPrompt) ||
+//  			 ((act == MergeActionNothing) &&
+//  			  (!option_force_resolution))))
 
 int MergeControl::compact (const MergeDef** arr,
 			   Mergeable* selected,
@@ -2123,7 +2158,7 @@ int MergeControl::compact (const MergeDef** arr,
     int nlen = 0;
 
     for (int i = 0; i < len; i += 1) {
-	if (arr[i] && !SKIP_FILE(file_merge_action(selected, common, working, arr[i]->rule_no))) {
+	if (arr[i]) { // && !SKIP_FILE(file_merge_action(selected, common, working, arr[i]->rule_no))) {
 	    arr[nlen++] = arr[i];
 	}
     }
@@ -2145,8 +2180,8 @@ PrVoidError MergeControl::run_merge (Mergeable* selected,
 
     for (int i = 0; i < ARRAY_LEN(merge_defaults); i += 1) {
 
-	if ((merge_defaults[i].have_working  == (working != NULL)) &&
-	    (merge_defaults[i].have_common   == (common != NULL)) &&
+	if ((merge_defaults[i].have_working  == (working  != NULL)) &&
+	    (merge_defaults[i].have_common   == (common   != NULL)) &&
 	    (merge_defaults[i].have_selected == (selected != NULL))) {
 
 	    found_defs[found++] = merge_defaults + i;
@@ -2172,18 +2207,12 @@ PrVoidError MergeControl::run_merge (Mergeable* selected,
 		Return_if_fail(ws_cmp << quick_cmp (selected, NULL, working, &ws_pos));
 	    if (common && selected)
 		Return_if_fail(sc_cmp << quick_cmp (selected, common, NULL, &sc_pos));
-#if 0 /* This never gets called because sc_cmp is always known. */
-	} else if (pass == 1 && selected && common && !sc_pos) {
-	    /* The SC comparison */
-	    Return_if_fail (checkout_unkey (selected, temp_file_selected,
-					    common, common_file_selected,
-					    NULL, NULL,
-					    &co));
 
-	    Return_if_fail(sc_cmp << file_cmp (temp_file_selected, temp_file_common));
+	} else if (pass == 1 && selected && common) {
+
+	    /* Nothing to do here, S-C is always positive */
+	    ASSERT (sc_pos, "not working, must be positive");
 
-	    sc_pos = true;
-#endif
 	} else if (pass == 2 && working && selected && !ws_pos) {
 	    /* The WS comparison */
 	    Return_if_fail (prepare (selected, temp_file_selected,
@@ -2197,9 +2226,9 @@ PrVoidError MergeControl::run_merge (Mergeable* selected,
 	} else if (pass == 3 && working && common && !wc_pos) {
 	    /* The WC comparison */
 	    Return_if_fail (prepare (NULL, NULL,
-						  common, temp_file_common,
-						  working, temp_file_working,
-						  &co));
+				     common, temp_file_common,
+				     working, temp_file_working,
+				     &co));
 
 	    Return_if_fail(wc_cmp << file_cmp (temp_file_common, temp_file_working));
 
@@ -2219,10 +2248,17 @@ PrVoidError MergeControl::run_merge (Mergeable* selected,
 	found = compact (found_defs, selected, common, working, found);
     }
 
-    if (found == 0 ||
-	file_merge_action (selected, common, working, found_defs[0]->rule_no) == (MergeAction)0) {
-	merge_announce (selected, common, working, (MergeAction)0);
-	merge_action   (selected, common, working, (MergeAction)0, 0, false);
+    ASSERT (found == 1, "no SKIP_FILE");
+
+    /* There was a thought that merge should treat NoPrompt actions
+     * differently by moving this if-statement into finish_merge(),
+     * see comment by default_merge_actions. */
+    if (file_merge_action (selected, common, working, found_defs[0]->rule_no) == MergeActionNoPrompt) {
+
+	int rule = (found == 0) ? 0 : found_defs[0]->rule_no;
+
+	merge_announce (selected, common, working, MergeActionNoPrompt);
+	merge_action   (selected, common, working, MergeActionNoPrompt, rule, false);
 
 	return NoError;
     }
