@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: populate.cc 1.11.1.6.1.12.1.3.1.8.1.21 Wed, 14 Oct 1998 20:50:34 -0700 jmacd $
+ * $Id: populate.cc 1.11.1.6.1.12.1.3.1.8.1.22 Wed, 24 Oct 2001 14:23:43 +0400 jmacd $
  */
 
 #include "prcs.h"
@@ -229,6 +229,10 @@ static int hash_dir(const char* dir, InoTable* T, PathTable *P)
 
 static bool heuristic_keyword_guess(const char* path)
 {
+    if (option_nokeywords) {
+	return true;
+    }
+
     FILE* file;
     char buffer[1024];
     int nread = 0;
