@@ -16,11 +16,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: prcs.cc 1.23.1.1.1.14.1.18.1.8.1.16.1.1.1.7.1.35 Sun, 31 Oct 1999 16:52:32 -0800 jmacd $
+ * $Id: prcs.cc 1.23.1.1.1.14.1.18.1.8.1.16.1.1.1.7.1.41.1.2 Thu, 07 Feb 2002 07:57:16 +0300 jmacd $
  */
 
 /* $Format: "static const char prcs_version_id[] = \"$ProjectVersion$ $ProjectAuthor$ $ProjectDate$\";"$ */
-static const char prcs_version_id[] = "1.2-release.94 jmacd Tue, 09 Nov 1999 14:40:19 -0800";
+static const char prcs_version_id[] = "1.3-release.14 jmacd Sat, 09 Feb 2002 05:02:20 +0300";
+
+#include "fnmatch.h"
 
 #include "prcs.h"
 #include "lock.h"
@@ -43,8 +45,11 @@ extern "C" {
 #include "getopt.h"
 }
 
+/* $Format: "const int prcs_version_number[3] = { $ReleaseMajorVersion$, $ReleaseMinorVersion$, $ReleaseMicroVersion$ };" $ */
+const int prcs_version_number[3] = { 1, 3, 1 };
+
 /* $Format: "const char prcs_version_string[] = \"$ReleaseVersion$\";"$ */
-const char prcs_version_string[] = "1.2.15";
+const char prcs_version_string[] = "1.3.1";
 
 /* The following classes are only used inside this file, and oraganize
  * several arrays of information used to select the command and
@@ -117,13 +122,17 @@ static struct CleanupHandler *cleanup_handler_list = NULL;
 static struct CleanupHandler *alarm_handler_list = NULL;
 
 static const char check_rcs_arg[] = "-V";
-static const char check_rcs_expected[] = "RCS version 5";
+static const char check_rcs_expected[] = "*RCS version 5*";
 static const CommandNamePair check_rcs = { &rcs_command, "rcs" };
 
 static const char check_diff_arg[] = "-v";
-static const char check_diff_expected[] = "diff - GNU diffutils version";
+static const char check_diff_expected[] = "*GNU diffutils*";
 static const CommandNamePair check_diff = { &gdiff_command, "gdiff" };
 
+static const char check_diff3_arg[] = "-v";
+static const char check_diff3_expected[] = "*GNU diffutils*";
+static const CommandNamePair check_diff3 = { &gdiff3_command, "gdiff3" };
+
 static CommandNamePair const command_names[] = {
     { &rcs_command, "rcs" },
     { &ci_command, "ci" },
@@ -144,9 +153,14 @@ static CommandNamePair const command_names[] = {
 #define PIPE_CHAR        '\304'
 #define PLAINFORMAT_CHAR '\305'
 #define SORT_CHAR        '\306'
+#define NOKEYW_CHAR      '\307'
+#define VLOG_CHAR        '\310'
+
 #define EXECUTE_STR      "\300\301\302\303\304"
 #define PLAINFORMAT_STR  "\305"
 #define SORT_STR         "\306"
+#define NOKEYW_STR       "\307"
+#define VLOG_STR         "\310"
 
 static CommandOptionPair const command_options[] = {
     { 'i', "--immediate", &option_immediate_uncompression },
@@ -158,6 +172,8 @@ static CommandOptionPair const command_options[] = {
     { 'P', "--exclude-project-file", &option_exclude_project_file },
     { 'u', "--unlink", &option_unlink },
     { 's', "--skilled-merge", &option_skilled_merge },
+    { NOKEYW_CHAR, "--no-keywords", &option_nokeywords },
+    { VLOG_CHAR, "--version-log", &option_version_log },
     { SORT_CHAR, "--sort", &option_sort },
     { MATCH_CHAR, "--match", &option_match_file },
     { NOTMATCH_CHAR, "--not", &option_not_match_file },
@@ -190,7 +206,6 @@ static struct option const long_options[] = {
   {"immediate",               no_argument, 0, 'i'},
   {"delete",                  no_argument, 0, 'd'},
   {"compress",                no_argument, 0, 'z'},
-  {"family",                  no_argument, 0, 'F'},
   {"preserve-permissions",    no_argument, 0, 'p'},
   {"preserve",                no_argument, 0, 'p'},
   {"permissions",             no_argument, 0, 'p'},
@@ -198,6 +213,8 @@ static struct option const long_options[] = {
   {"revision",                required_argument, 0, 'r'},
   {"unlink",                  no_argument, 0, 'u'},
   {"skilled-merge",           no_argument, 0, 's'},
+  {"no-keywords",             no_argument, 0, NOKEYW_CHAR},
+  {"version-log",             required_argument, 0, VLOG_CHAR},
   {"sort",                    required_argument, 0, SORT_CHAR},
   {"match",                   required_argument, 0, MATCH_CHAR},
   {"not",                     required_argument, 0, NOTMATCH_CHAR},
@@ -206,8 +223,10 @@ static struct option const long_options[] = {
   {"pipe",                    no_argument, 0, PIPE_CHAR},
   {0,0,0,0}};
 
-static const char prcs_optstring[] = "+vhHLlnqfR:r:NkidzFpPruj:s"
-                                     EXECUTE_STR PLAINFORMAT_STR SORT_STR ":"
+/* It looks like a bug in the optstring that EXECUTE_STR members don't
+ * have ':' characters, but GNU getopt doesn't seem to mind. */
+static const char prcs_optstring[] = "+vhHLlnqfR:r:NkidzpPruj:s"
+                                     EXECUTE_STR PLAINFORMAT_STR NOKEYW_STR VLOG_STR ":" SORT_STR ":"
 #ifdef PRCS_DEVEL
 "Dt:"
 #endif
@@ -217,7 +236,7 @@ static PrcsCommand commands[] = {
     /* commands accepting -r */
     {"checkin", "", "@", "@", NoFlags,
      checkin_command, checkin_help_string,
-     OneRevisionArg, InsureProjectName, "uj", NoDiffArgs, -1 },
+     OneRevisionArg, InsureProjectName, "uj" VLOG_STR, NoDiffArgs, -1 },
 
     {"checkout", "@", "@", "@", NoFlags,
      checkout_command, checkout_help_string,
@@ -225,11 +244,11 @@ static PrcsCommand commands[] = {
 
     {"diff", "", "", "@", NoFlags,
      diff_command, diff_help_string,
-     TwoRevisionArgs, InsureProjectName, "NkFP", ReallyDiffArgs, -1 },
+     TwoRevisionArgs, InsureProjectName, "NkP", ReallyDiffArgs, -1 },
 
     {"merge", "", "@", "@", NoFlags,
      merge_command, merge_help_string,
-     OneRevisionArg, InsureProjectName, "Fus", ReallyDiffArgs, -1 },
+     OneRevisionArg, InsureProjectName, "us", ReallyDiffArgs, -1 },
 
     {"delete", "", "", "@", NoDefault,
      delete_command, delete_help_string,
@@ -239,6 +258,10 @@ static PrcsCommand commands[] = {
      info_command, info_help_string,
      OneRevisionArg, InsureProjectName, SORT_STR, NoDiffArgs, -1 },
 
+    {"changes", "", "", "@", NoFlags,
+     changes_command, changes_help_string,
+     TwoRevisionArgs, InsureProjectName, "", NoDiffArgs, -1 },
+
     {"execute", "", "", "@", NoFlags,
      execute_command, execute_help_string,
      OneRevisionArg, InsureProjectName, "P" EXECUTE_STR, ReallyExecuteArgs, -1 },
@@ -246,7 +269,7 @@ static PrcsCommand commands[] = {
     /* commands not accepting -r */
     {"populate", NULL, NULL, NULL, NoFlags,
      populate_command, populate_help_string,
-     NoRevisionArg, InsureProjectName, "du", NoDiffArgs, -1 },
+     NoRevisionArg, InsureProjectName, "du" NOKEYW_STR, NoDiffArgs, -1 },
 
     {"depopulate", NULL, NULL, NULL, NoFlags,
      depopulate_command, depopulate_help_string,
@@ -358,7 +381,7 @@ static PrVoidError check_system_command(CommandNamePair com,
 			<< dotendl;
     }
 
-    if(strncmp(output.cast(), expected, strlen (expected)) != 0) {
+    if (fnmatch (expected, output.cast (), FNM_NOESCAPE) != 0) {
 	pthrow prcserror << com.name
 			 << " installation is too old, please install "
 			 << expected << dotendl;
@@ -377,6 +400,10 @@ static PrVoidError check_system_commands()
 					check_diff_arg,
 					check_diff_expected));
 
+    Return_if_fail(check_system_command(check_diff3,
+					check_diff3_arg,
+					check_diff3_expected));
+
     return NoError;
 }
 
@@ -447,7 +474,7 @@ PrPrcsExitStatusError config_command()
     prcsoutput << "Plain formatting is: " << setcol(width)
 	       << (option_plain_format ? "on" : "off") << prcsendl;
 
-    Return_if_fail(check_system_commands());
+    //Return_if_fail(check_system_commands());
 
     return ret;
 }
@@ -814,7 +841,7 @@ static bool read_command_line(int argc, char** argv)
 	char *min;
 	switch (c) {
 	case 'v':
-	    prcsoutput << "Version " << prcs_version_string << endl;
+	    prcsoutput << "Version " << prcs_version_string << prcsendl;
 	    exit(0);
 	    break;
 	case 'h': case 'H': return false; break;
@@ -856,6 +883,8 @@ static bool read_command_line(int argc, char** argv)
 	case PIPE_CHAR: option_pipe = 1; break;
 	case PLAINFORMAT_CHAR: option_plain_format = 1; break;
 	case SORT_CHAR: option_sort = 1; option_sort_type = optarg; break;
+	case NOKEYW_CHAR: option_nokeywords = 1; break;
+	case VLOG_CHAR: option_version_log = 1; option_version_log_string = optarg; break;
 #ifdef PRCS_DEVEL
 	case 'D': option_n_debug = 0; break;
 	case 't': option_tune = atoi (optarg); break;
@@ -1006,20 +1035,20 @@ static bool read_command_line(int argc, char** argv)
 static void usage(char** argv)
 {
     if (illegal_option) {
-	cerr << options_summary << endl;
+	cout << options_summary << "\n";
     } else if(command == NULL) {
-	cerr << "Usage: " << strip_leading_path(argv[0])
+	cout << "Usage: " << strip_leading_path(argv[0])
 	     << " command [subcommand] [option ...] "
 	    "[project [file-or-dir]]\n";
-	cerr << general_help_string;
+	cout << general_help_string;
     } else if(strcmp(command->command_name, "admin") == 0) {
 	if(subcommand == NULL) {
-	    cerr << admin_help_string;
+	    cout << admin_help_string;
 	} else {
-	    cerr << subcommand->help_string;
+	    cout << subcommand->help_string;
 	}
     } else {
-	cerr << command->help_string;
+	cout << command->help_string;
     }
 
     exit(2);
@@ -1291,10 +1320,15 @@ int main(int argc, char** argv)
 	exit(2);
     }
 
-    if(subcommand == NULL)
+    If_fail(check_system_commands()) {
+	exit(2);
+    }
+
+    if(subcommand == NULL) {
 	exitval = invoke_command(command);
-    else
+    } else { 
 	exitval = invoke_command(subcommand);
+    }
 
     bool clean_failed = false;
 
@@ -1302,8 +1336,9 @@ int main(int argc, char** argv)
 	clean_failed = true;
 
     if(Failure(exitval) || clean_failed) {
-	if(exitval.error_val() != UserAbort || clean_failed)
+	if(exitval.error_val() != UserAbort || clean_failed) {
 	    prcserror << "Command failed" << dotendl;
+	}
 	return 2;
     }
 
