@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: prcserror.cc 1.8.1.11.1.3.1.7.1.11 Tue, 29 Sep 1998 13:26:59 -0700 jmacd $
+ * $Id: prcserror.cc 1.8.1.11.1.3.1.7.1.11.2.1 Mon, 04 Feb 2002 00:39:38 +0300 jmacd $
  */
 
 
@@ -546,7 +546,7 @@ ostream& QueryOstream::string_query_manip(const char* message)
 	    if (!ans) ans = new Dstring;
 
 	    If_fail(read_string(stdin, ans)) {
-		cout << endl;
+		cout << "\n";
 		string_val = PrConstCharPtrError(UserAbort);
 		break;
 	    }
@@ -669,7 +669,7 @@ ostream& QueryOstream::query_manip(const char* message)
 	    if(c == '\n') { /* User accepts default. */
 		c = options[default_option].let;
 	    } else if(c == '\0') { /* EOF. */
-		cout << endl;
+		cout << "\n";
 		val = PrCharError(UserAbort);
 		break;
 	    } else if(c == '?') { /* User needs help. */
