@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: rebuild.cc 1.3.1.1.1.8.1.21.1.4.1.4.1.12 Tue, 06 Oct 1998 13:31:13 -0700 jmacd $
+ * $Id: rebuild.cc 1.3.1.1.1.8.1.21.1.4.1.4.1.14.1.4 Thu, 07 Feb 2002 07:57:16 +0300 jmacd $
  */
 
 extern "C" {
@@ -125,8 +125,10 @@ static PrVoidError compute_cksum(const char* versionfile, RcsVersionData* ver)
 
     Return_if_fail(cofile << VC_checkout_stream(ver->rcs_version(), versionfile));
 
-    If_fail(read_buf.map_file(fileno(cofile)))
-	pthrow prcserror << "Read failed from RCS co pipe" << dotendl;
+    // Blah
+    Dstring ds;
+    describe_versionfile (versionfile, ver, ds);
+    Return_if_fail(read_buf.map_file(ds.cast(), fileno(cofile)));
 
     ver->length(read_buf.length());
 
@@ -161,28 +163,30 @@ PrVoidError create_empty_data(const char* project, const char* filename0)
 /* Write out the ProjectVersionData structure */
 static void write_data_file_project_info(ostream& os, ProjectVersionData* ver)
 {
-    os << ver->date() << ends;
-    os << ver->author() << ends;
-    os << ver->rcs_version() << ends;
+    os << ver->date() << '\0';
+    os << ver->author() << '\0';
+    os << ver->rcs_version() << '\0';
 
-    os << (int)ver->deleted() << ends;
+    os << (int)ver->deleted() << '\0';
 
-    os << ver->prcs_major() << ends
-       << ver->prcs_minor() << ends;
+    os << ver->prcs_major() << '\0'
+       << ver->prcs_minor() << '\0';
 
-    os << ver->parent_count() << ends;
+    os << ver->parent_count() << '\0';
 
     for (int i = 0; i < ver->parent_count(); i += 1)
-	os << ver->parent_index(i) << ends;
+	os << ver->parent_index(i) << '\0';
 }
 
 /* Write out the RcsVersionData structure */
 static void write_data_file_rcs_info(ostream& os, RcsVersionData* ver)
 {
-    os << ver->date() << ends;
-    os << ver->length() << ends;
-    os << ver->author() << ends;
-    os << ver->rcs_version() << ends;
+    os << ver->date() << '\0';
+    os << ver->length() << '\0';
+    os << ver->author() << '\0';
+    os << ver->rcs_version() << '\0';
+    os << ver->plus_lines () << '\0';
+    os << ver->minus_lines () << '\0';
 
     os.write(ver->unkeyed_checksum(), 16);
 }
@@ -191,7 +195,7 @@ static void write_data_file_rcs_info(ostream& os, RcsVersionData* ver)
 static void write_data_file_project_summary(ostream& os,
 					    ProjectVersionDataPtrArray *project_summary)
 {
-    os << project_summary_header << project_summary->length() << ends;
+    os << project_summary_header << project_summary->length() << '\0';
 
     foreach(proj_ptr2, project_summary, ProjectVersionDataPtrArray::ArrayIterator)
 	write_data_file_project_info(os, *proj_ptr2);
@@ -203,8 +207,8 @@ static void write_data_file_rcs_file_summary(ostream&os, const char* name, RcsDe
     /* Here because reference_files isn't removing files it deletes
      * from the table and this is easier. */
 
-    os << rcs_file_summary_header << name << ends;
-    os << version_count_header << delta->count() << ends;
+    os << rcs_file_summary_header << name << '\0';
+    os << version_count_header << delta->count() << '\0';
 
     foreach(ver, delta, RcsDelta::DeltaIterator)
 	write_data_file_rcs_info(os, *ver);
@@ -267,7 +271,7 @@ static PrVoidError ignoring(ProjectVersionData* project_data)
     return NoError;
 }
 
-/* Mark all versions referenced by the a single version of a project
+/* Mark all versions referenced by a single version of a project
  * in the file table. */
 static PrVoidError reference_versions(const char* project_file_name,
 				      ProjectVersionData* project_data,
@@ -335,6 +339,8 @@ static PrVoidError reference_versions(const char* project_file_name,
 	version_data->reference(RcsVersionData::Referenced);
     }
 
+    /* @@@ This is where the more-complete parent-version consistency check should go. */
+
     delete project;
 
     return NoError;
@@ -405,6 +411,8 @@ static PrRcsDeltaPtrError rebuild_repository_file(RepEntry* rep_entry,
 	return (RcsDelta*)0;
     }
 
+    /* @@@ Ideally, the cksum would get stored in the RCS log message so that this step
+     * could actually verify, not simply re-compute the cksum. */
     foreach(rev_ptr, data, RcsDelta::DeltaIterator) {
 	Return_if_fail(compute_cksum(name, *rev_ptr));
     }
@@ -472,16 +480,95 @@ static PrRcsFileTablePtrError rebuild_file_table(RepEntry* rep_ptr)
     return file_table;
 }
 
+/* This mainly checks for duplicate version numbers, a symptom of the
+ * bug reported by Keith Owens in versions up to 1.3.0.  Also checks
+ * for invalid parent-version indices. */
+static PrVoidError rebuild_check_project_summary (ProjectVersionDataPtrArray* project_summary)
+{
+    ProjectVersionDataPtrArray copy (*project_summary);
 
-/* This is the top-level call to rebuilding a repository data file. */
+    /* Loop through a copy of the project summary, once for each major
+     * version, checking for duplicate minor version numbers.  (Note:
+     * similarly ugly code to info_array() -- need to simplify this
+     * kind of iteration) */
+    for (int i = 0; i < copy.length (); i += 1) {
+
+	if (copy.index (i) != NULL) {
+
+	    const char *major = copy.index(i)->prcs_major ();
+	    int         minor = 0;
+
+	    for (int j = i; j < copy.length (); j += 1) {
+
+		ProjectVersionData *check = copy.index(j);
+
+		if (check == NULL) {
+		    continue;
+		}
+
+		if (strcmp (major, check->prcs_major ()) == 0) {
+
+		    int this_minor = check->prcs_minor_int ();
+
+		    if (this_minor <= minor) {
+			pthrow prcserror << ((this_minor <= 0) ? "Invalid" : "Duplicate")
+					 << " project version "
+					 << check
+					 << " found, please contact "
+					 << maintainer
+					 << " for assistance with this problem"
+					 << dotendl;
+		    }
+
+		    while (++minor < this_minor) {
+			/* I made this a warning not an error simply because there is a
+			 * missing version in PRCS's own repository (from Feb 20, 1996,
+			 * and I don't remember why).  If this is a genuine error it needs
+			 * to be caught somewhere else.  A check should be inserted in
+			 * reference_versions when the project file is actually
+			 * checked-out that the parent-indices match reality. */
+			prcswarning << "Missing any record of project version "
+				    << major << "." << minor
+				    << prcsendl;
+		    }
+
+		    ASSERT (check->version_index () == j, "set by VC_get_project_version_data");
+
+		    for (int k = 0; k < check->parent_count (); k += 1) {
+
+			int p_i = check->parent_index (k);
+
+			if (p_i < 0 || p_i >= j) {
+			    pthrow prcserror << "Invalid parent-version index found for project version "
+					     << check << ", please contact "
+					     << maintainer
+					     << " for assitance with this problem"
+					     << dotendl;
+			}
+		    }
+
+		    copy.index (j, (ProjectVersionData*)0);
+		}
+	    }
+	}
+    }
+
+    return NoError;
+}
+
+/* This is the top-level call to rebuild a repository data file. */
 static PrVoidError rebuild_repository(RepEntry *rep_ptr,
 				      ProjectVersionDataPtrArray* project_summary,
 				      RcsFileTable* file_table)
 {
-    if(!file_table || !project_summary) {
+    if(!project_summary) {
 	Return_if_fail(project_summary <<
 		       VC_get_project_version_data(rep_ptr->Rep_name_of_version_file()));
+    }
 
+    Return_if_fail(rebuild_check_project_summary (project_summary));
+
+    if(!file_table) {
 	Return_if_fail(file_table << rebuild_file_table(rep_ptr));
     }
 
@@ -543,6 +630,8 @@ PrVoidError admin_rebuild_command_no_open(RepEntry* rep, bool valid_rep_data)
 
     Return_if_fail(rep->Rep_compress_all_files());
 
+    Return_if_fail(rep->Rep_make_default_tag ());
+
     return NoError;
 }
 
@@ -597,7 +686,7 @@ PrPrcsExitStatusError delete_command()
 						   NULL,
 						   rep_entry));
 #if 0
-    @@@@
+    /*@@@ I have no idea about this*/
     if(project_data->prcs_minor_int() == 0)
 	pthrow prcserror << "You may not delete minor version 0" << dotendl;
 #endif
@@ -705,6 +794,7 @@ PrRcsVersionDataPtrError RebuildFile::read_rcs_version()
 {
     RcsVersionData* ver = new RcsVersionData;
     const char* cksum;
+    const char *pls, *mls;
 
     ver->date(get_size());
     ver->length(get_size());
@@ -718,6 +808,12 @@ PrRcsVersionDataPtrError RebuildFile::read_rcs_version()
     if(!(ver->author() && ver->rcs_version()))
 	return bad_data_file();
 
+    if (! (pls = get_string ()) || ! (mls = get_string ()))
+	return bad_data_file ();
+
+    ver->set_plus_lines (pls);
+    ver->set_minus_lines (mls);
+
     if(!(cksum = get_string(16)))
 	return bad_data_file();
 
@@ -797,9 +893,7 @@ PrVoidError RebuildFile::init_from_file(const char* filename0, bool write)
 
     seg = new MemorySegment(false, write);
 
-    If_fail(seg->map_file(filename0))
-	pthrow prcserror << "Failed opening project data file "
-			<< squote(filename0) << perror;
+    Return_if_fail(seg->map_file(filename0));
 
     last = seg->segment() + seg->length();
     offset = 0;
@@ -857,7 +951,7 @@ bool RebuildFile::get_string(const char* expected, bool term)
     int len = strlen(expected);
 
     if(last - (seg->segment() + offset) < len || seg->length() < offset)
-	return NULL;
+	return false;
 
     if(strncmp(expected, seg->segment() + offset, len) != 0)
 	return false;
@@ -897,12 +991,12 @@ bool RebuildFile::done() { return offset == seg->length(); }
 void RebuildFile::init_stream()
 {
     if(!buf) {
-#ifdef __GNUG__
+#if __GNUG__ < 3
 	buf = new filebuf(seg->fd());
 	buf->seekoff(0,ios::end);
 #else
-        buf = new filebuf(fdopen(dup(seg->fd()), "a+"));
-        buf->pubseekoff(0,ios::end, ios::out);
+        buf = new filebuf(fdopen(dup(seg->fd()), "a+"), ios::out);
+        buf->pubseekoff(0, ios::end, ios::out);
 #endif
 	os = new ostream(buf);
     }
@@ -913,7 +1007,7 @@ void RebuildFile::add_project_data(ProjectVersionData* data)
 {
     init_stream();
 
-    (*os) << project_summary_header << 1 << ends;
+    (*os) << project_summary_header << 1 << '\0';
 
     write_data_file_project_info(*os, data);
 }
@@ -922,8 +1016,8 @@ void RebuildFile::add_rcs_file_data(const char* new_file_name, RcsVersionData* d
 {
     init_stream();
 
-    (*os) << rcs_file_summary_header << new_file_name << ends;
-    (*os) << version_count_header << 1 << ends;
+    (*os) << rcs_file_summary_header << new_file_name << '\0';
+    (*os) << version_count_header << 1 << '\0';
 
     write_data_file_rcs_info(*os, data);
 }
@@ -951,7 +1045,7 @@ PrVoidError RebuildFile::update_project_data()
 
 error:
     pthrow prcserror << "Failed updating repository data file"
-		    << squote(source_name()) << perror;
+		     << squote(source_name()) << perror;
 }
 
 RebuildFile::~RebuildFile()
